"""
Data module for handling financial data acquisition, processing, and management.
"""

from .timeseries import TimeSeries, TimeSeriesMetadata
from .multitimeseries import MultiTimeSeries
from .ratiotimeseries import RatioTimeSeries
from .spreadtimeseries import SpreadTimeSeries

__all__ = [
    'TimeSeries',
    'TimeSeriesMetadata',
    'MultiTimeSeries',
    'RatioTimeSeries',
    'SpreadTimeSeries'
] 